/*! 
 * Collection
 *
 * Copyright (C) 2011-2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */
 
"use strict";

var io = require("socket.io");
var SocketIoManager = require("socket.io").Manager;
var Static = require("socket.io").Static;

/**
 * Connect the Collections library to a socket.
 * 
 * @param app: an HTTPServer instance, Express app, port number, or existing socket.io instance
 * @param options: optional options.
 *
 * @return a socket.io manager
 */
exports.listen = function listen(app, options) {

    if (!(app instanceof SocketIoManager)) {
        app = io.listen(app, options);
    }
    
    return new exports.Manager(app, options);
}

exports.Manager = require("./manager");

exports.Store = require("./store");

exports.Memory = require("./stores/memory");

exports.MongoDb = require("./stores/mongodb");

exports.Redis = require("./stores/redis");

// Merge all errors
var errors = require("./errors");
for (var error in errors) {
    exports[error] = errors[error];
}