/*! 
 * Collection
 *
 * Copyright (C) 2011-2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */
 
"use strict";

var util = require("util");

/**
 * This error is thrown if the user input failed validation.
 *
 * @api public
 */
var ValidationError = exports.ValidationError = function(id, fields, message) {
    this.message = message || "Validation Error";
    
    this.id = id;
    this.fields = fields;
    
    Error.captureStackTrace(this, ValidationError);
}

util.inherits(ValidationError, Error);

/**
 * This error is thrown if a user-specified ID is invalid.
 *
 * @api public
 */
var InvalidIdError = exports.InvalidIdError = function(id, message) {
    this.id = id;
    this.message = message || "Invalid Id Error";
    
    Error.captureStackTrace(this, InvalidIdError);
}

util.inherits(InvalidIdError, Error);

/**
 * This error is thrown if a read, update or delete request is used with
 * a non-existing object.
 *
 * @api public
 */
var NotFoundError = exports.NotFoundError = function(id, message) {
    this.id = id;
    this.message = message || "Id Not Found";
    
    Error.captureStackTrace(this, NotFoundError);
}

util.inherits(NotFoundError, Error);

/**
 * This error is thrown if an user request is malformed.
 *
 */
var MalformedRequestError = exports.MalformedRequestError = function(message) {
    this.message = message;
    
    Error.captureStackTrace(this, MalformedRequestError);
}

util.inherits(MalformedRequestError, Error);