/*! 
 * Collection
 *
 * Copyright (C) 2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */

 
"use strict";

var collection = require("../collection"),
    util = require("util"),
    Store = require("../store"),
    errors = require("../errors");

var Memory = module.exports = function(options) {
    var self = this;
    
    this.settings = {
        
    }
    Store.apply(this, arguments);

    if (this.params.length) {
        this.array = {};
    } else {
        this.array = [];
    }
    
    this.lastId = 0;
}

util.inherits(Memory, Store);

/** 
 *
 */
Memory.prototype.getArray = function(params) {
    var i, l;
    var currentArray = this.array;
    
    for (i = 0, l = this.params.length; i < l; ++i) {
        var paramName = this.params[i];
        if (typeof currentArray[params[paramName]] === "undefined") {
            if (i == l-1) {
                currentArray[params[paramName]] = [];
            } else {
                currentArray[params[paramName]] = {};
            }
        }
        
        currentArray = currentArray[params[paramName]];
    }
    
    return currentArray;
}

/** 
 *
 */
Memory.prototype.doInsert = function(objects, id, key, revision, data) {
    var i = 0;

    id = id || this.lastId++;
    
    for (i = 0; i < objects.length; ++i) {
        if (key < objects[i].key)
            break;
    }

    objects.splice(i, 0, {
        id: id,
        key: key,
        revision: revision,
        data: data
    });

    return objects[i];
}

/** 
 *
 */
Memory.prototype.doDelete = function(objects, id) {
    var i, l;
    
    for (i = 0, l = objects.length; i < l; i++) {
        if (id == objects[i].id)
            break;
    }
    if (i >= objects.length) {
        return null;
    }
    
    var descriptor = objects[i];
    
    objects.splice(i, 1);
    
    return descriptor; 
}

/** 
 *
 */
Memory.prototype.read = function(req, res) {
    var objects = this.getArray(req.params);
    var descriptor = [];
    var counter = 0;
    
    if (req.direction < 0) {
        var reversedObjects = objects.reverse();
        
        for (var i in reversedObjects) {
            if (reversedObjects[i].key <= req.key) {
                descriptor.push(reversedObjects[i]);
                ++counter;
            }
            if (counter >= req.count) {
                break;
            }
        }
        
        descriptor = descriptor.reverse();
    } else {
        for (i in objects) {
            if (objects[i].key >= req.key) {
                descriptor.push(objects[i]);
                ++counter;
            }
            if (counter >= req.count) {
                break;
            }
        }
    }
    
    res(null, descriptor);
}

/** 
 *
 */
Memory.prototype.insert = function(req, res) {
    var objects = this.getArray(req.params);
    
    var descriptor = this.doInsert(objects, null, req.key, 0, req.data);

    res(null, descriptor);
}

/** 
 *
 */
Memory.prototype.delete = function(req, res) {
    var objects = this.getArray(req.params);
    
    var descriptor = this.doDelete(objects, req.id);
    
    if (descriptor) {
        res(null, descriptor);
    } else {
        res(new errors.NotFoundError(req.id));
    }
}

/** 
 *
 */
Memory.prototype.update = function(req, res) {
    var objects = this.getArray(req.params);
    var descriptor = this.doDelete(objects, req.id);
    
    if (!descriptor) {
        res(new errors.NotFoundError(req.id));
    } else {
        if (typeof req.key === "number") {
            descriptor.key = req.key;
        }
        descriptor.data = req.data;
        descriptor = this.doInsert(objects, descriptor.id, descriptor.key, descriptor.revision + 1, descriptor.data);
        res(null, descriptor);
    }
}

