/*! 
 * Collection
 *
 * Copyright (C) 2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */

/**
 * Module dependencies
 */
var collectionIo = require("../.."),
    http = require("http"),
    fs = require("fs"),
    path = require("path"),
    url = require("url"),
    mongodb = require("mongodb"),
    async = require("async");

var staticPath = __dirname + "/static/";

// A basic static-file server
var server = http.createServer(function (req, res) {
    var reqUrl = url.parse(req.url);
    var reqPath = path.normalize(staticPath + (reqUrl.pathname === "/" ? "/index.html" : reqUrl.pathname));
    
    console.log("Request for " + reqUrl.pathname);
    console.log(reqPath);
    
    if (reqPath.indexOf(staticPath) !== 0) {
        // Hey!
        res.writeHead(400);
        res.end();
    } else {
        path.exists(reqPath, function(exists) {
            if (exists) {
                fs.readFile(reqPath, function(err, content) {
                    if (err) {
                        res.writeHead(500);
                        res.end();
                    } else {
                        var contentType;
                        
                        switch (path.extname(reqPath).toLowerCase()) {
                        case ".html":
                            contentType = "text/html";
                            break;
                        case ".js":
                            contentType =  "application/javascript";
                            break;
                        case ".css":
                            contentType = 'text/css';
                            break;
                        default:
                            contentType = "text/plain";
                            break;
                        }
                        
                        contentType += ";charset=utf-8";
                        
                        switch (req.method) {
                        case "HEAD":
                            res.writeHead(200, { "Content-Type": contentType });
                            res.end();
                            break;
                        case "GET":
                            res.writeHead(200, { "Content-Type": contentType });
                            res.end(content, "utf-8");
                            break;
                        default:
                            res.writeHead(405);
                            res.end();
                            break;
                        }
                    }
                });
            } else {
                res.writeHead(404);
                res.end();
            }
        });
    }
});

var collectionManager = collectionIo.listen(server);
var store = {};

collectionManager.add('mongo', new collectionIo.MongoDb({ db: "test", collection: "test" }));
collectionManager.add('mongop', new collectionIo.MongoDb({ db: "test", collection: "testp", params: ["p"] }));
collectionManager.add('memory', new collectionIo.Memory());
collectionManager.add('redis', new collectionIo.Redis({ key: "test" }));

console.log("Starting the server");
server.listen(3000);






