/*! 
 * Collection
 *
 * Copyright (C) 2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */


require(
    ["dojo/_base/connect", "dojo/ready", "dojo/query", "dojo/NodeList-manipulate", "dojo/parser", "dojo/dom-construct", "dojo/dom-geometry", "dojo/dom-attr", "dojo/dom-style", "dojo/_base/fx", "dojo/fx", "dijit/form/Button", "dijit/form/TextBox", "dijit/form/Textarea", "dijit/form/RadioButton"],
    function(connect, ready, query, nlm, parser, domConstruct, domGeom, domAttr, domStyle, fx, coreFx, Button, TextBox, Textarea, RadioButton) {
        
        function CollectionViewer(id) {
            this.id = id;
            this.itemArray = [];
        }
        
        CollectionViewer.prototype.setNodeContent = function (node, data) {
            domAttr.set(node, "data-id", data.id);
            domAttr.set(node, "data-key", data.key);
            domAttr.set(node, "data-data", data.data.text);
            node.innerHTML = '<span class="node-key">' + data.key + '</span>' + data.data.text;
        }
        
        CollectionViewer.prototype.createNode = function (data) {
            var node = dojo.create("div", {
                "class": "item", 
                style: "height: 0; margin-top: 0; margin-bottom: 0; padding-top: 0; padding-bottom: 0" 
            });
            
            this.setNodeContent(node, data);
            
            dojo.connect(node, "onclick", function() {
                dijit.byId("keyInput").set("value", domAttr.get(this, "data-key"));
                dijit.byId("idInput").set("value", domAttr.get(this, "data-id"));
                dijit.byId("dataInput").set("value", domAttr.get(this, "data-data"));
            });
            return node;
        }
        
        CollectionViewer.prototype.placeNode = function (node, to) {
            if (to >= this.itemArray.length) {
                domConstruct.place(node, dojo.byId(this.id), "last");
            } else {
                domConstruct.place(node, this.itemArray[to].node, "before");
            }
        }
        
        CollectionViewer.prototype.changeHandler = function(from, to, data) {
            var node;
            var anim1, anim2, anim3;
            var self = this;
            
            var log = dojo.byId("log");
            domConstruct.place("<div>onChange(" + from + ", " + to + ", "+ data+")</div>", log, "last");
            log.scrollTop = log.scrollHeight;
            
            
            if (from >= 0 && to < 0) {
                var fakeNode = dojo.create("div", {
                    "class": "item", 
                    style: "visibility: hidden; display: none" 
                });
                this.placeNode(fakeNode, from);
                var node = this.itemArray[from].node;
                anim1 = fx.animateProperty({
                    node: node,
                    properties: {
                        opacity: 0
                    },
                
                    onEnd: function() {
                        domStyle.set(fakeNode, "display", "block");
                        domConstruct.destroy(node);
                    }
                });
                
                anim2 = fx.animateProperty({
                    node: fakeNode,
                    properties:  { height: { start: 30, end: 0 }, marginTop: 0, marginBottom: 0, paddingTop: 0, paddingBottom: 0 },
                    onEnd: function() {
                        domConstruct.destroy(fakeNode);
                    }
                });
                
                this.itemArray.splice(from, 1);
                
                var anim = coreFx.chain([anim1, anim2]);
                anim.play();
            }
            if (from < 0 && to >= 0) {
                var node = this.createNode(data);
                
                data.node = node;
                this.placeNode(node, to);
                fx.animateProperty({node: node, properties: { height: 30, marginTop: 5, marginBottom: 5, paddingTop: 10, paddingBottom: 10 }}).play();
                this.itemArray.splice(to, 0, data);
            }
            if (from >= 0 && to >= 0) {
                node = this.itemArray[from].node;
                
                if (from !== to) {
                    // Insert two invisible placeholder items
                    var fakeNode1 = dojo.create("div", {
                        "class": "item", 
                        style: "visibility: hidden" 
                    });
                    
                    var fakeNode2 = dojo.create("div", {
                        "class": "item", 
                        style: "height: 0; margin-top: 0; margin-bottom: 0; padding-top: 0; padding-bottom: 0; visibility: hidden" 
                    });
            
                    var startPos = domGeom.position(node);
                    domStyle.set(node, "position", "absolute");
                    
                    this.itemArray.splice(from, 1);
                    this.placeNode(fakeNode2, to);
                    var goalPos = domGeom.position(fakeNode2).y;
                    this.placeNode(fakeNode1, from);
                    this.placeNode(node, to);
                    data.node = node;
                    this.itemArray.splice(to, 0, data);
                    domStyle.set(node, "top", (startPos.y - domStyle.get(node, "margin-top")) +"px");
                    domStyle.set(node, "left", (startPos.x - domStyle.get(node, "margin-left")) +"px");
                    
                    anim1 = fx.animateProperty({
                        node: fakeNode1,
                        properties: { height: { start: 30, end: 0 }, marginTop: 0, marginBottom: 0, paddingTop: 0, paddingBottom: 0 } 
                    });
                    
                    anim2 = fx.animateProperty({
                        node: fakeNode2,
            
                        properties: { height: 30, marginTop: 5, marginBottom: 5, paddingTop: 10, paddingBottom: 10 }
                    });
                    
                    anim3 = fx.animateProperty({
                        node: node,
                        properties: { top: goalPos }
                    });
                    
                    var anim = coreFx.combine([anim1, anim2, anim3]);
                    
                    connect.connect(anim, "onEnd", function() {
                        domConstruct.destroy(fakeNode1);
                        domConstruct.destroy(fakeNode2);
                        domStyle.set(node, "position", "relative");
                        domStyle.set(node, "top", "0");
                        domStyle.set(node, "left", "0");
                    });   
                    
                    anim.play();
                }
                
                this.setNodeContent(node, data);
            }
        }
        
        CollectionViewer.prototype.errorHandler = function(error) {
            domConstruct.place("<div>onError(" + error +")</div>", log, "last");
        }
        
        CollectionViewer.prototype.setCollection = function(coll) {
            if (this.collection) {
                this.destroyCollection();
            }
            
            var self = this;
            this.collection = coll;
            
            this.collection.onChange = function(from, to, data) {
                self.changeHandler(from, to, data);
            }
            
            this.collection.onError = function(error) {
                self.errorHandler(error);
            }
        }
        
        CollectionViewer.prototype.getCollection = function(coll) {
            return this.collection;
        }
        
        CollectionViewer.prototype.destroyCollection = function() {
            this.collection.destroy();
            delete this.collection;
            
            this.itemArray = [];
            query("#" + this.id + " .item").remove();
        }
        
        var getSelectedColl = function () {
            var selected0 = dijit.byId("collSelect0").get("checked");
            var selected1 = dijit.byId("collSelect1").get("checked");
            var selected2 = dijit.byId("collSelect2").get("checked");
            
            var selected;
            
            if (selected0) {
                selected = 0;
            } else if (selected1) {
                selected = 1;
            } else if (selected2) {
                selected = 2;
            }
            
            return selected;
        }
        
        ready(function() {
            var urlInput = dijit.byId("urlInput");
            urlInput.set("value", document.location.origin + "/");
        
            var insertButton = dijit.byId("insertButton");
            var updateButton = dijit.byId("updateButton");
            var deleteButton = dijit.byId("deleteButton");
            
            var viewers = [
                new CollectionViewer("collection-1"),
                new CollectionViewer("collection-2"),
                new CollectionViewer("collection-3")
            ];
                    
            dojo.connect(dijit.byId("connectButton"), "onClick", function() {
                var url = dijit.byId("urlInput").get("value");
                var collName = dijit.byId("collInput").get("value");
                var paramsValue = dijit.byId("paramInput").get("value");
                var params = paramsValue ? JSON.parse(paramsValue) : null;
                
                var coll = new collection.Collection(url, collName, { params: params });
                viewers[getSelectedColl()].setCollection(coll);
                
                insertButton.set("disabled", false);
                updateButton.set("disabled", false);
                deleteButton.set("disabled", false);
            });
            
            dojo.connect(dijit.byId("disconnectButton"), "onClick", function() {
                insertButton.set("disabled", true);
                updateButton.set("disabled", true);
                deleteButton.set("disabled", true);
                
                viewers[getSelectedColl()].destroyCollection();
            });
            
            dojo.connect(insertButton, "onClick", function() {
                var coll = viewers[getSelectedColl()].getCollection();
                coll.insert(
                    dijit.byId("keyInput").get("value"),
                    { text: dijit.byId("dataInput").get("value") }
                );
            });
            dojo.connect(updateButton, "onClick", function() {
                var coll = viewers[getSelectedColl()].getCollection();
                coll.update(
                    dijit.byId("idInput").get("value"),
                    dijit.byId("keyInput").get("value"),
                    { text: dijit.byId("dataInput").get("value") }
                );
            });
            dojo.connect(deleteButton, "onClick", function() {
                var coll = viewers[getSelectedColl()].getCollection();
                coll["delete"](dijit.byId("idInput").get("value"));
            });
        
        });
    }
);