/*! 
 * Collection
 *
 * Copyright (C) 2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */

var util = require("util"),
    CollectionStore = require("../../../lib/collection").Store;

var Logger = exports.Logger = function() {

}

Logger.prototype.error = function() {}
Logger.prototype.warn = function() {}
Logger.prototype.info = function() {}
Logger.prototype.debug = function() {}


var Io = exports.Io = function(callback) {
    if (callback) {
        this.callback = callback;
    } else {
        this.callback = function() {};
    }
    this.log = new Logger;
}

Io.prototype = {
    of: function() { return this },
    on: function() {},
    emit: function(message, data) {
        this.callback(null, { message: message, data: data });
    },
    connect: function(server) {
        return new Socket(this.callback);
    }
};


var Socket = exports.Socket = function(callback) {
    this.operations = [];
    this.flags = {};
    
    if (callback) {
        this.callback = callback;
    } else {
        this.callback = function() {};
    }
}

Socket.prototype = {
    get broadcast() {
        this.flags.broadcast = true;
        return this;
    },
    to: function(room) {
        this.flags.room = room;
        return this;
    },
    emit: function(message, data) {
        this.flags.message = message;
        this.flags.data = data;
        this.operations.push(this.flags);
        
        if (!this.flags.broadcast)
            this.callback(null, this.flags);
        this.flags = {};
    },
    join: function(room) {
        this.flags.join = room;
        this.operations.push(this.flags);
        this.callback(null, this.flags);
        this.flags = {};
    },
    leave: function(room) {
        this.flags.leave = room;
        this.operations.push(this.flags);
        this.callback(null, this.flags);
        this.flags = {};
    },
    on: function() {
    
    }
};

var Store = exports.Store = function(options, callback) {
    if (typeof options === "function") {
        callback = options;
        options = {};
    }
    if (callback) {
        this.callback = callback;
    } else {
        this.callback = function () {};
    }

    CollectionStore.call(this, options);
};

util.inherits(Store, CollectionStore);

Store.prototype.read = function(req, res) {
    this.operation = { op: "read", req: req, res: res };
    this.callback(null, this.operation);
};
    
Store.prototype.insert = function(req, res) {
    this.operation = { op: "insert", req: req, res: res };
    this.callback(null, this.operation);
};

Store.prototype.delete = function(req, res) {
    this.operation = { op: "delete", req: req, res: res };
    this.callback(null, this.operation);
};
    
Store.prototype.update = function(req, res) {
    this.operation = { op: "update", req: req, res: res };
    this.callback(null, this.operation);
};

var Res = exports.Res = function(callback) {
    return function(err, descriptor) {
        if (err) {
            callback(null, {
                error: err
            });
        } else {
            callback(null, descriptor);
        }
    }
};


