/*! 
 * Collection
 *
 * Copyright (C) 2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */

var basedir;

if (process.env.JSCOVERAGE) {
    basedir = "../../var/jscoverage";
} else {
    basedir = "../../lib";
}

var vows = require("vows"),
    mock = require("./lib/mock"),
    assert = require("assert"),
    Memory = require(basedir + "/stores/memory"),
    async = require("async");
    
    
vows.describe("Memory")
.addBatch({
    "Insert one object": {
        topic: function() {
            var self = this;
            var memory = new Memory();
            
            async.series([
                function(callback) {
                    memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "Called send with an insert descriptor": function(err, res) {
            assert.equal(typeof res.descriptors[0], "object");
        },
        
        "The data actually was inserted in the array": function(err, res) {
            assert.equal(res.data.length, 1);
            assert.equal(res.data[0].key, 0);
            assert.equal(res.data[0].data.t, "Test0");
        }

    },
    "Insert 2 objects": {
        topic: function() {
            var self = this;
            var memory = new Memory();
            
            async.series([
                function(callback) {
                    memory.insert({ key: 1, data: { t: "Test1" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "2 inserts called": function(err, res) {
            assert.equal(res.descriptors.length, 2);
        },
        
        "The data in the array is correct and ordered": function(err, res) {
            assert.equal(res.data.length, 2);
            assert.equal(res.data[0].key, 0);
            assert.equal(res.data[0].data.t, "Test0");
            assert.equal(res.data[1].key, 1);
            assert.equal(res.data[1].data.t, "Test1");
        }
    },
    "Insert 2 objects, then invert positions": {
        topic: function() {
            var self = this;
            var memory = new Memory();
            
            async.series([
                function(callback) {
                    memory.insert({ key: 1, data: { t: "Test1" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.update({ id: memory.array[0].id, key: 2, data: { t: "Test0" }}, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "3 descriptors": function(err, res) {
            assert.equal(res.descriptors.length, 3);
        },
        
        "The 3rd descriptor updates the object to the last pos": function(err, res) {
            assert.equal(res.descriptors[2].id, 1);
            assert.equal(res.descriptors[2].key, 2);
        },
        
        "There are still 2 items": function(err, res) {
            assert.equal(res.data.length, 2);
        },
        
        "The data in the array is correct and ordered": function(err, res) {
            assert.equal(res.data[0].key, 1);
            assert.equal(res.data[0].data.t, "Test1");
            assert.equal(res.data[1].key, 2);
            assert.equal(res.data[1].data.t, "Test0");
        }
    },
    "Insert 3 objects, then delete the middle one": {
        topic: function() {
            var self = this;
            var memory = new Memory();
            
            async.series([
                function(callback) {
                    memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 1, data: { t: "Test1" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 2, data: { t: "Test2" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.delete({ id: memory.array[1].id }, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "4 descriptors": function(err, res) {
            assert.equal(res.descriptors.length, 4);
        },
        
        "There are 2 items": function(err, res) {
            assert.equal(res.data.length, 2);
        },
        
        "The data in the array is correct and ordered": function(err, res) {
            assert.equal(res.data[0].key, 0);
            assert.equal(res.data[0].data.t, "Test0");
            assert.equal(res.data[1].key, 2);
            assert.equal(res.data[1].data.t, "Test2");
        }
    },
    "Insert 3 objects, then delete the middle one, sent as string": {
        topic: function() {
            var self = this;
            var memory = new Memory();
            
            async.series([
                function(callback) {
                    memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 1, data: { t: "Test1" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 2, data: { t: "Test2" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.delete({ id: "" + memory.array[1].id }, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "4 descriptors": function(err, res) {
            assert.equal(res.descriptors.length, 4);
        },
        
        "There are 2 items": function(err, res) {
            assert.equal(res.data.length, 2);
        },
        
        "The data in the array is correct and ordered": function(err, res) {
            assert.equal(res.data[0].key, 0);
            assert.equal(res.data[0].data.t, "Test0");
            assert.equal(res.data[1].key, 2);
            assert.equal(res.data[1].data.t, "Test2");
        }
    },
    "Insert 3 objects, read": {
        topic: function() {
            var self = this;
            var memory = new Memory();
            
            async.series([
                function(callback) {
                    memory.insert({ key: 2, data: { t: "Test2" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 1, data: { t: "Test1" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
                },
                function(callback) {
                    memory.read({ key: 1, count: 2 }, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "The data in the array is correct and ordered": function(err, res) {
            var lastDesc = res.descriptors.pop();
            
            assert.equal(lastDesc.length, 2);
            assert.equal(lastDesc[0].key, 1);
            assert.equal(lastDesc[0].data.t, "Test1");
            assert.equal(lastDesc[1].key, 2);
            assert.equal(lastDesc[1].data.t, "Test2");
        }
    },
    "Insert 3 objects, read inverted": {
       topic: function() {
           var self = this;
           var memory = new Memory();
           
           async.series([
               function(callback) {
                   memory.insert({ key: 2, data: { t: "Test2" }}, new mock.Res(callback));
               },
               function(callback) {
                   memory.insert({ key: 1, data: { t: "Test1" }}, new mock.Res(callback));
               },
               function(callback) {
                   memory.insert({ key: 0, data: { t: "Test0" }}, new mock.Res(callback));
               },
               function(callback) {
                   memory.read({ key: 1, count: 2, direction: -1 }, new mock.Res(callback));
               }
           ], function(err, descriptors) {
               self.callback(err, { descriptors: descriptors, data: memory.array });
           });
               
       },
       
       "The data in the array is correct and ordered": function(err, res) {
           var lastDesc = res.descriptors.pop();
           
           assert.equal(lastDesc.length, 2);
           assert.equal(lastDesc[0].key, 0);
           assert.equal(lastDesc[0].data.t, "Test0");
           assert.equal(lastDesc[1].key, 1);
           assert.equal(lastDesc[1].data.t, "Test1");
       }
    },
    "Use of params": {
        topic: function() {
            var self = this;
            var memory = new Memory({ params: ["foo", "bar" ]});
            
            async.series([
                function(callback) {
                    memory.insert({
                        key: 0,
                        params: { 
                            foo: 1, 
                            bar: 2 
                        }, 
                        data: { t: "Test0" }
                    }, new mock.Res(callback));
                }
            ], function(err, descriptors) {
                self.callback(err, { descriptors: descriptors, data: memory.array });
            });
                
        },
        
        "Called send with an insert descriptor": function(err, res) {
            assert.equal(typeof res.descriptors[0], "object");
        },
        
        "The hierarchical structure of the array is correct": function(err, res) {
            assert.equal(typeof res.data, "object");
            assert.equal(typeof res.data[1], "object");
            assert(res.data[1][2] instanceof Array);
            
        },
        "The data actually was inserted in the array": function(err, res) {

            assert.equal(res.data[1][2].length, 1);
            assert.equal(res.data[1][2][0].key, 0);
            assert.equal(res.data[1][2][0].data.t, "Test0");
        }

    },
}).export(module);