/*! 
 * Collection
 *
 * Copyright (C) 2012 Philippe Plantier <philippe@midsummerweb.com>
 *
 * MIT Licensed
 */


var basedir;

if (process.env.JSCOVERAGE) {
    basedir = "../../var/jscoverage";
} else {
    basedir = "../../lib";
}

var vows = require("vows"),
    mock = require("./lib/mock"),
    assert = require("assert"),
    MongoDb = require(basedir + "/stores/mongodb"),
    mongodb = require("mongodb"),
    async = require("async");
    
    
vows.describe("MongoDb")
.addBatch({
    "Create a store using only text parameters": {
        topic: function() {
            return new MongoDb({
                host: "localhost",
                db: "test",
                collection: "test"
            });
        },
        
        "Did not fail": function(err, store) {
            assert.ok(!(store instanceof Error));
        },
        
        "No error": function(err, store) {
            assert.ifError(store.error);
        }
    },
    "Create a store an open DB": {
        topic: function() {
            var self = this;
            
            var db = new mongodb.Db(
                "test",
                new mongodb.Server("localhost", 27017, {})
            );
            
            db.open(function(err, p_client) {
                self.callback(err, new MongoDb({
                    db: db,
                    collection: "test"
                }));
            });
        },
        "Did not fail": function(err, store) {
            assert.ok(!(store instanceof Error));
        },
        "No error": function(err, store) {
            assert.ifError(store.error);
        }
        
    },
    "Create a store using an open collection": {
        topic: function() {
            var self = this;
            
            var db = new mongodb.Db(
                "test",
                new mongodb.Server("localhost", 27017, {})
            );
            
            db.open(function(err, p_client) {
                if (err) {
                    self.callback(err);
                    return;
                }
                
                db.collection("test", function(err, collection) {
                    self.callback(err, new MongoDb({
                        db: db,
                        collection: collection
                    }));
                });
            });
        },
        "Did not fail": function(err, store) {
            assert.ok(!(store instanceof Error));
        },
        "No error": function(err, store) {
            assert.ifError(store.error);
        }

    },
    "Create a store with no parameters (should fail)": {
        topic: function() {
            return new MongoDb({});
        },
        "Did not fail": function(err, store) {
            assert.ok(!(store instanceof Error));
        },
        "There is an error": function(err, store) {
            assert.ok(store.error);
        }

    }
}).export(module);